{-# OPTIONS_HADDOCK show-extensions #-}

-- |
-- Module      :  Phladiprelio.Ukrainian.PropertiesFuncRepG2Diverse
-- Copyright   :  (c) Oleksandr Zhabenko 2020-2023
-- License     :  MIT
-- Stability   :  Experimental
-- Maintainer  :  oleksandr.zhabenko@yahoo.com
--
-- Generalization of the functionality of the DobutokO.Poetry.Norms
-- and DobutokO.Poetry.Norms.Extended modules
-- from the @dobutokO-poetry@ package. Instead of vectors, uses arrays.

{-# LANGUAGE NoImplicitPrelude, MagicHash #-}

module Phladiprelio.Ukrainian.PropertiesFuncRepG2Diverse (
  -- * Functions with 'Int16'
  procDiverse2ITup
  -- ** Usual ones
  , procDiverse2FTup
) where

import GHC.Enum (fromEnum)
import GHC.Base
import GHC.Int
import GHC.Arr (Array)
import Phladiprelio.UniquenessPeriodsG
import Phladiprelio.Basis
import GHC.Float (int2Double)
import Phladiprelio.Ukrainian.Melodics
import Phladiprelio.Ukrainian.Emphasis

procDiverse2ITup
  :: (Ord c) => Array Int (Int8, Bool)
  -> Array Int (Int8, Bool)
  -> Array Int (Int8, Bool)
  -> Array Int (Int8, Bool)
  -> Array Int ([Int8], Bool)
  -> Array Int ([Int8], Int8)
  -> Array Int (Int8, FlowSound -> Sound8)
  -> Array Int (Int8, Bool)
  -> Array Int ([Int8], Bool)
  -> Array Int ([Int8], Bool)
  -> Array Int ([Int8], Bool)
  -> Array Int (Int8, [Int8])
  -> Array Int (Char,Int8)
  -> Array Int (Int8,[Int8])
  -> Array Int (Char, Bool)
  -> Array Int (Char, Bool)
  -> (Int16 -> c)
  -> FlowSound
  -> FuncRep2 ReadyForConstructionUkr Int16 c
procDiverse2ITup tup1 tup2 tup3 tup4 tup5 tup6 tup7 tup8 tup9 tup10 tup11 tup12 tup13 tup14 tup15 tup16 g sels
  = D (\x -> case x of
               Str xs -> diverse2GGL sels [100,101] .
                  convertToProperUkrainianI8WithTuples tup1 tup2 tup3 tup4 tup5 tup6 tup7 tup8 tup9 tup10 tup11 tup12 tup13 tup14 tup15 tup16 $ xs
               FSL xsss -> 1) g
{-# INLINE procDiverse2ITup #-}

procDiverse2FTup
  :: (Ord c) => Array Int (Int8, Bool)
  -> Array Int (Int8, Bool)
  -> Array Int (Int8, Bool)
  -> Array Int (Int8, Bool)
  -> Array Int ([Int8], Bool)
  -> Array Int ([Int8], Int8)
  -> Array Int (Int8, FlowSound -> Sound8)
  -> Array Int (Int8, Bool)
  -> Array Int ([Int8], Bool)
  -> Array Int ([Int8], Bool)
  -> Array Int ([Int8], Bool)
  -> Array Int (Int8, [Int8])
  -> Array Int (Char,Int8)
  -> Array Int (Int8,[Int8])
  -> Array Int (Char, Bool)
  -> Array Int (Char, Bool)
  -> (Double -> c)
  -> FlowSound
  -> FuncRep2 ReadyForConstructionUkr Double c
procDiverse2FTup tup1 tup2 tup3 tup4 tup5 tup6 tup7 tup8 tup9 tup10 tup11 tup12 tup13 tup14 tup15 tup16 g sels
  = D (\x -> case x of
               Str xs -> int2Double . fromEnum . diverse2GGL sels [100,101] .
                  convertToProperUkrainianI8WithTuples tup1 tup2 tup3 tup4 tup5 tup6 tup7 tup8 tup9 tup10 tup11 tup12 tup13 tup14 tup15 tup16 $ xs
               FSL xsss -> 1.0) g
{-# INLINE procDiverse2FTup #-}
